resource "aws_api_gateway_rest_api" "this" {
  name = "${local.resource_prefix}-api-gateway"
}

resource "aws_api_gateway_deployment" "this" {
  rest_api_id = aws_api_gateway_rest_api.this.id

  lifecycle {
    create_before_destroy = true
    replace_triggered_by = [
      null_resource.always_run
    ]
  }

  depends_on = [
    aws_api_gateway_method.hello_world_get,
    aws_api_gateway_integration.hello_world_get,
    aws_api_gateway_method.hello_world_options,
    aws_api_gateway_integration.hello_world_options,
    # aws_acm_certificate.ssl_certificate, aws_route53_record.cert_validation,
    # aws_acm_certificate_validation.ssl_certificate_validation, aws_api_gateway_domain_name.this,
    # aws_route53_record.api, aws_api_gateway_gateway_response.unauthorized_response, module.apigateway_resources,
  ]
}

resource "aws_api_gateway_stage" "this" {
  stage_name            = var.environment
  rest_api_id           = aws_api_gateway_rest_api.this.id
  deployment_id         = aws_api_gateway_deployment.this.id
  documentation_version = aws_api_gateway_documentation_version.this.version

  xray_tracing_enabled = var.xray_tracing_enabled

  tags = {
    Name = local.resource_prefix
  }

  depends_on = [
    aws_api_gateway_method.hello_world_get,
    aws_api_gateway_integration.hello_world_get,
    aws_api_gateway_method.hello_world_options,
    aws_api_gateway_integration.hello_world_options,
    aws_api_gateway_documentation_version.this,
    # aws_acm_certificate.ssl_certificate, aws_route53_record.cert_validation,
    # aws_acm_certificate_validation.ssl_certificate_validation, aws_api_gateway_domain_name.this,
    # aws_route53_record.api, aws_api_gateway_gateway_response.unauthorized_response, module.apigateway_resources
  ]
}

resource "aws_api_gateway_method_settings" "this" {
  rest_api_id = aws_api_gateway_rest_api.this.id
  stage_name  = aws_api_gateway_stage.this.stage_name
  method_path = "*/*"

  settings {
    throttling_burst_limit = var.api_throttling_burst_limit
    throttling_rate_limit  = var.api_throttling_rate_limit
    metrics_enabled        = var.api_metrics_enabled
    logging_level          = var.api_logging_level
    data_trace_enabled     = var.api_data_trace_enabled
  }
}

resource "aws_iam_role" "cloudwatch" {
  name = "api_gateway_cloudwatch_global"

  assume_role_policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "",
      "Effect": "Allow",
      "Principal": {
        "Service": "apigateway.amazonaws.com"
      },
      "Action": "sts:AssumeRole"
    }
  ]
}
EOF
}

resource "aws_iam_role_policy" "cloudwatch" {
  name = "default"
  role = aws_iam_role.cloudwatch.id

  policy = <<EOF
{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Effect": "Allow",
            "Action": [
                "logs:CreateLogGroup",
                "logs:CreateLogStream",
                "logs:DescribeLogGroups",
                "logs:DescribeLogStreams",
                "logs:PutLogEvents",
                "logs:GetLogEvents",
                "logs:FilterLogEvents"
            ],
            "Resource": "*"
        }
    ]
}
EOF
}

resource "aws_api_gateway_account" "this" {
  cloudwatch_role_arn = aws_iam_role.cloudwatch.arn
}

resource "aws_api_gateway_authorizer" "this" {
  name          = "${local.resource_prefix}-authorizer"
  type          = "COGNITO_USER_POOLS"
  rest_api_id   = aws_api_gateway_rest_api.this.id
  provider_arns = ["${aws_cognito_user_pool.pool.arn}"]
}

resource "aws_api_gateway_documentation_part" "api" {
  location {
    type = "API"
  }

  properties  = "{\"description\":\"Test REST API Gateway deployed with terraform\"}"
  rest_api_id = aws_api_gateway_rest_api.this.id
}

resource "aws_api_gateway_documentation_version" "this" {
  version     = "latest"
  rest_api_id = aws_api_gateway_rest_api.this.id
  description = "API documentation"
  depends_on = [
    aws_api_gateway_documentation_part.api,
    aws_api_gateway_documentation_part.hello_world
  ]
}

resource "null_resource" "openapi_export" {
  lifecycle {
    replace_triggered_by = [
      null_resource.always_run
    ]
  }

  depends_on = [
    aws_api_gateway_stage.this
  ]

  provisioner "local-exec" {
    command = "aws apigateway get-export --parameters extensions='apigateway' --rest-api-id ${aws_api_gateway_rest_api.this.id} --stage-name ${aws_api_gateway_stage.this.stage_name} --export-type oas30 --output yaml ../../doc/api.yml"
  }
}